﻿/*
      Copyright (C) 2009 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using System.Diagnostics;

namespace BasicSetupDemo
{
    public class Camera
    {
        public Vector3 position;
        float myYaw;
        float myPitch;
        public float Yaw
        {
            get
            {
                return myYaw;
            }
            set
            {
                myYaw = value;
                if (myYaw >= MathHelper.Pi * 2)
                    myYaw -= MathHelper.Pi * 2;
                else if (myYaw < 0)
                    myYaw += MathHelper.Pi * 2;
            }
        }
        public float Pitch
        {
            get
            {
                return myPitch;
            }
            set
            {
                myPitch = value;
                if (myPitch >= MathHelper.PiOver2)
                    myPitch = MathHelper.PiOver2;
                else if (myPitch < -MathHelper.PiOver2)
                    myPitch = -MathHelper.PiOver2;
            }
        }
        public float speed;
        Matrix myViewMatrix;
        public Matrix ViewMatrix
        {
            get
            {
                return myViewMatrix;
            }
        }
        public Matrix projectionMatrix;

        Matrix myWorldMatrix;
        public Matrix WorldMatrix
        {
            get
            {
                return myWorldMatrix;
            }
        }

        public Camera(Vector3 position, float yaw, float pitch, float speed)
        {
            this.position = position;
            myYaw = yaw;
            myPitch = pitch;
            this.speed = speed;
            projectionMatrix = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, 4f/3f, .1f, 10000.0f);
            Mouse.SetPosition(200, 200);
        }

        public void MoveForward(float dt)
        {
            position += myWorldMatrix.Forward * (dt * speed);
        }

        public void MoveRight(float dt)
        {
            position += myWorldMatrix.Right * (dt * speed);
        }

        public void MoveUp(float dt)
        {
            position += new Vector3(0, (dt * speed), 0);
        }

        public void Update(float dt)
        {
            myViewMatrix = Matrix.CreateTranslation(-position) * Matrix.CreateFromAxisAngle(Vector3.Up, -Yaw) * Matrix.CreateFromAxisAngle(Vector3.Right, -Pitch);
            myWorldMatrix = Matrix.Invert(myViewMatrix);
        }

    }
}
