﻿/*
      Copyright (C) 2009 Bepu Entertainment LLC

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework;
using BEPUphysics.Entities;

namespace BasicSetupDemo
{
    public class User : GameComponent
    {
        public Camera camera;

        public User(Camera camera, BowlingGame game)
            : base(game)
        {
            this.camera = camera;
        }

        public override void Update(GameTime gameTime)
        {
            float dt = (float)gameTime.ElapsedGameTime.TotalSeconds;
            camera.Update(dt);
            HandleInput(dt);
            base.Update(gameTime);
        }

        public void HandleInput(float dt)
        {
#if XBOX360
            BowlingGame game = Game as BowlingGame;
            GamePadState gamePadState = game.gamePadState;

            if (gamePadState.Triggers.Right > 0)
            {
                //If the user is holding down the trigger, start firing some boxes.
                //First, create a new dynamic box at the camera's location.
                Box toAdd = new Box(camera.position, 1, 1, 1, 1);
                //Set the velocity of the new box to fly in the direction the camera is pointing.
                //Entities have a whole bunch of properties that can be read from and written to.
                //Try looking around in the entity's available properties to get an idea of what is available.
                toAdd.linearVelocity = camera.WorldMatrix.Forward * 10;
                //Add the new box to the simulation.
                game.space.add(toAdd);
            
                //Add a graphical representation of the box to the drawable game components.
                game.Components.Add(new EntityModel(toAdd, game.cubeModel, Matrix.Identity, game));
            }

            //Scoot the camera around depending on what gamepad input exists.
            camera.Yaw += gamePadState.ThumbSticks.Right.X * -.025f;
            camera.Pitch += gamePadState.ThumbSticks.Right.Y * .025f;
            camera.MoveForward(dt * gamePadState.ThumbSticks.Left.Y);
            camera.MoveRight(dt * gamePadState.ThumbSticks.Left.X);
            if (gamePadState.IsButtonDown(Buttons.LeftStick))
                camera.MoveUp(dt);
            if (gamePadState.IsButtonDown(Buttons.RightStick))
                camera.MoveUp(-dt);
#else
            BowlingGame game = Game as BowlingGame;
            KeyboardState keyboardState = game.keyboardState;
            MouseState mouseState = game.mouseState;

            float xDifference = mouseState.X - 200;
            float yDifference = mouseState.Y - 200;
            camera.Yaw -= xDifference * .002f;
            camera.Pitch -= yDifference * .002f;
            Mouse.SetPosition(200, 200);


            if (mouseState.LeftButton == ButtonState.Pressed)
            {
                //If the user is clicking, start firing some boxes.
                //First, create a new dynamic box at the camera's location.
                Sphere toAdd = new Sphere(camera.position, .3f, 8);
                //Set the velocity of the new box to fly in the direction the camera is pointing.
                //Entities have a whole bunch of properties that can be read from and written to.
                //Try looking around in the entity's available properties to get an idea of what is available.
                toAdd.linearVelocity = camera.WorldMatrix.Forward * 10;
                //Add the new box to the simulation.
                game.space.add(toAdd);

                //Add a graphical representation of the box to the drawable game components.
                game.Components.Add(new EntityModel(toAdd, game.bowlingBallModel, Matrix.CreateScale(.7f), game));
            }

            //Scoot the camera around depending on what keys are pressed.
            if (keyboardState.IsKeyDown(Keys.E))
                camera.MoveForward(dt);
            if (keyboardState.IsKeyDown(Keys.D))
                camera.MoveForward(-dt);
            if (keyboardState.IsKeyDown(Keys.F))
                camera.MoveRight(dt);
            if (keyboardState.IsKeyDown(Keys.S))
                camera.MoveRight(-dt);
            if (keyboardState.IsKeyDown(Keys.A))
                camera.MoveUp(dt);
            if (keyboardState.IsKeyDown(Keys.Z))
                camera.MoveUp(-dt);
#endif

        }
    }
}
